MODULE DTC;

IMPORT WinBase;
IMPORT IO;
IMPORT Lib3, Str, StrOut, Config, Clock, Utils;
(*
IMPORT ClockTst;
*)

TYPE
  TWeekDayStr = ARRAY [0..2] OF CHAR;
  TWeekDayStrs = ARRAY Clock.TWeekDay OF TWeekDayStr;

CONST
  wdStrs = TWeekDayStrs{ 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun' };

CONST
  SO ::= StrOut;

VAR
  chr: CHAR;
  pos: CARDINAL;
  str: Config.TArgStr;
  len: CARDINAL;
  idx: CARDINAL;


PROCEDURE WrDate( VAR b: SO.TBuffer; date: Clock.TDate );
VAR
  c: CHAR;
  y, m, d: CARDINAL;
BEGIN
  Clock.UnpackDate( date, y, m, d );
  c := b.c.p;
  b.c.p := '0';
  SO.WrCard( b, y, 4 );
  SO.WrChar( b, '-' );
  SO.WrCard( b, m, 2 );
  SO.WrChar( b, '-' );
  SO.WrCard( b, d, 2 );
  b.c.p := c;
END WrDate;

PROCEDURE WrTime( VAR b: SO.TBuffer; time: Clock.TTime );
VAR
  c: CHAR;
  h, m, s, f: CARDINAL;
BEGIN
  Clock.UnpackTime( time, h, m, s, f );
  c := b.c.p;
  b.c.p := '0';
  SO.WrCard( b, h, 2 );
  SO.WrChar( b, ':' );
  SO.WrCard( b, m, 2 );
  SO.WrChar( b, ':' );
  SO.WrCard( b, s, 2 );
  SO.WrChar( b, '.' );
  SO.WrCard( b, f, 3 );
  b.c.p := c;
END WrTime;


PROCEDURE ShowDateTime( d: Clock.TDate; t: Clock.TTime );
VAR
  b: SO.TBuffer;
  s: ARRAY [0..127] OF CHAR;
BEGIN
  SO.Init( b, s );
  WrDate( b, d );
  SO.WrChar( b, ' ' );
  WrTime( b, t );

  b.c.p := '0';
  SO.WrStr( b, '  ' );
  SO.WrHex( b, d, 4 );
  SO.WrStr( b, 'H.' );
  SO.WrLngHex( b, t, 8 );
  SO.WrStr( b, 'H (' );
  SO.WrCard( b, d, 1 );
  SO.WrChar( b, '.' );
  SO.WrLngCard( b, t, 1 );
  SO.WrChar( b, ')' );

  SO.WrStr( b, '  ' );
  SO.WrStr( b, wdStrs[ Clock.WeekDay( d ) ] );
  SO.WrStr( b, ' CW ' );
  SO.WrCard( b, Clock.CalWeek( d ), 2 );
  IO.WrStr( s );
  IO.WrLn();
END ShowDateTime;


PROCEDURE ReadChr();
BEGIN
  IF pos < Config.data.len THEN
    chr := Config.data.arg[ pos ];
    INC( pos );
  ELSE
    chr := 0C;
  END;
END ReadChr;

PROCEDURE Base( VAR s: ARRAY OF CHAR ): CARDINAL;
VAR
  l: CARDINAL;
BEGIN
  l := Str.Length( s );
  IF l > 0 THEN
    CASE CAP( s[l-1] ) OF
    | 'H': s[l-1] := 0C; RETURN 16;
    | 'D': s[l-1] := 0C; RETURN 10;
    | 'O': s[l-1] := 0C; RETURN  8;
    | 'B': s[l-1] := 0C; RETURN  2;
    ELSE
    END;
  END;
  RETURN 10;
END Base;

PROCEDURE Read();
CONST
  seps = ' .,;-:+/|';
BEGIN
  Str.ItemS( str, Config.data.arg, seps, idx );
  INC( idx );
  len := Str.Length( str );
  Str.Caps( str );
END Read;



PROCEDURE Err( p-: ARRAY OF CHAR );
VAR
  b: SO.TBuffer;
  s: ARRAY [0..127] OF CHAR;
BEGIN
  SO.Init( b, s );
  SO.WrStr( b, 'ERROR: ' );
  SO.WrStr( b, p );
  SO.WrStr( b, ': "' );
  SO.WrStr( b, str );
  SO.WrStr( b, '"' );
  Lib3.FatalError( s );
END Err;


PROCEDURE Now();
VAR
  date: Clock.TDate;
  time: Clock.TTime;
  t: WinBase.SYSTEMTIME;
BEGIN
  (*
  WinBase.GetLocalTime( t );
  *)
  WinBase.GetSystemTime( t );
  date := Clock.PackDate( t.wYear, t.wMonth, t.wDay );
  time := Clock.PackTime( t.wHour, t.wMinute, t.wSecond, t.wMilliseconds );
  ShowDateTime( date, time );
END Now;


PROCEDURE Unpack();
VAR
  date: Clock.TDate;
  time: Clock.TTime;
  x: LONGCARD;
  ok: BOOLEAN;
BEGIN
  date := MIN( Clock.TDate );
  time := MIN( Clock.TTime );

  Read();
  IF len > 0 THEN
    x := Str.StrToCard( str, Base( str ), ok );
    IF ~ ok OR ( x > MAX(Clock.TDate) ) THEN Err( 'Invalid machine date' ) END;
    date := Clock.TDate( x );
  END;

  Read();
  IF len > 0 THEN
    x := Str.StrToCard( str, Base( str ), ok );
    IF ~ ok OR ( x > Clock.maxTime ) THEN Err( 'Invalid machine time' ) END;
    time := Clock.TTime( x );
  END;

  ShowDateTime( date, time );

  Read();
  IF len > 0 THEN Err( 'Unexpected argument' ) END;
END Unpack;


PROCEDURE Pack();
VAR
  idx: CARDINAL;

  PROCEDURE Rd( n: CARDINAL; VAR ok: BOOLEAN ): CARDINAL;
  VAR
    v: CARDINAL;
    c: CHAR;
  BEGIN
    v := 0;
    WHILE n > 0 DO
      c := CAP( str[ idx ] );
      INC( idx );
      ok := ( '0' <= c ) & ( c <= '9' );
      IF ~ ok THEN RETURN 0 END;
      v := v  * 10 + ORD( c ) - ORD( '0' );
      DEC( n );
    END;
    ok := TRUE;
    RETURN v;
  END Rd;

VAR
  date: Clock.TDate;
  time: Clock.TTime;
  year, month, day,
  hour, minute, second, frag: CARDINAL;
  x: LONGCARD;
  ok: BOOLEAN;
BEGIN
  year  := Clock.minYear;
  month := Clock.minMonth;
  day   := Clock.minDay;
  hour   := 0;
  minute := 0;
  second := 0;
  frag   := 0;

  Read();
  IF len > 4 THEN
    (* format YYYYMMDD *)
    IF len # 8 THEN Err( 'Invalid YYYYMMDD' ) END;
    idx := 0;
    x := Rd( 4, ok );
    IF ~ ok OR ( x < Clock.minYear ) OR ( x > Clock.maxYear ) THEN Err( 'Invalid year' ) END;
    year := x;
    x := Rd( 2, ok );
    IF ~ ok OR ( x < Clock.minMonth ) OR ( x > Clock.maxMonth ) THEN Err( 'Invalid month' ) END;
    month := x;
    x := Rd( 2, ok );
    IF ~ ok OR ( x < Clock.minDay ) OR ( x > Clock.maxDay ) THEN Err( 'Invalid day' ) END;
    day := x;
    IF ~ Clock.IsValidDate( year, month, day ) THEN Err( 'Invalid date' ) END;
  ELSE
    (* format Y M D *)
    IF len > 0 THEN
      x := Str.StrToCard( str, Base( str ), ok );
      IF ~ ok OR ( x < Clock.minYear ) OR ( x > Clock.maxYear ) THEN Err( 'Invalid year' ) END;
      year := x;
    END;
    Read();
    IF len > 0 THEN
      x := Str.StrToCard( str, Base( str ), ok );
      IF ~ ok OR ( x < Clock.minMonth ) OR ( x > Clock.maxMonth ) THEN Err( 'Invalid month' ) END;
      month := x;
      IF ~ Clock.IsValidDate( year, month, 1 ) THEN Err( 'Invalid month for year' ) END;
    END;
    Read();
    IF len > 0 THEN
      x := Str.StrToCard( str, Base( str ), ok );
      IF ~ ok OR ( x < Clock.minDay ) OR ( x > Clock.maxDay ) THEN Err( 'Invalid day' ) END;
      day := x;
      IF ~ Clock.IsValidDate( year, month, day ) THEN Err( 'Invalid day for year and month' ) END;
    END;
  END;
  date := Clock.PackDate( year, month, day );

  Read();
  IF len > 2 THEN
    (* format HHMMDD *)
    IF len # 6 THEN Err( 'Invalid HHMMSS' ) END;
    idx := 0;
    x := Rd( 2, ok );
    IF ~ ok OR ( x >= 24 ) THEN Err( 'Invalid hour' ) END;
    hour := x;
    x := Rd( 2, ok );
    IF ~ ok OR ( x >= 60 ) THEN Err( 'Invalid minute' ) END;
    minute := x;
    x := Rd( 2, ok );
    IF ~ ok OR ( x >= 60 ) THEN Err( 'Invalid second' ) END;
    second := x;
    Read();
    IF len > 0 THEN
      x := Str.StrToCard( str, Base( str ), ok );
      IF ~ ok OR ( x >= 1000 ) THEN Err( 'Invalid fragment' ) END;
      frag := x;
    END;
  ELSE
    (* format H M D *)
    IF len > 0 THEN
      x := Str.StrToCard( str, Base( str ), ok );
      IF ~ ok OR ( x >= 24 ) THEN Err( 'Invalid hour' ) END;
      hour := x;
    END;
    Read();
    IF len > 0 THEN
      x := Str.StrToCard( str, Base( str ), ok );
      IF ~ ok OR ( x >= 60 ) THEN Err( 'Invalid minute' ) END;
      minute := x;
    END;
    Read();
    IF len > 0 THEN
      x := Str.StrToCard( str, Base( str ), ok );
      IF ~ ok OR ( x >= 60 ) THEN Err( 'Invalid second' ) END;
      second := x;
    END;
    Read();
    IF len > 0 THEN
      x := Str.StrToCard( str, Base( str ), ok );
      IF ~ ok OR ( x >= 1000 ) THEN Err( 'Invalid fragment' ) END;
      frag := x;
    END;
  END;
  time := Clock.PackTime( hour, minute, second, frag );

  ShowDateTime( date, time );

  Read();
  IF len > 0 THEN Err( 'Unexpected argument' ) END;
END Pack;


BEGIN
  chr := 0C;
  pos := 0;
  str := '';
  len := 0;
  idx := 0;

  IF Config.data.len = 0 THEN Now() ELSIF Config.data.mch THEN Unpack() ELSE Pack() END;
  Lib3.SetReturnCode( 0 );
END DTC.
